import os
import tensorflow as tf
import numpy as np
from tensorflow.keras.models import load_model

print('TensorFlow Version: ', tf.__version__)
print('Numpy Version: ', np.__version__)

def Menu():
    '''
    :return: Type of Coding, code Configurations list, Channel Type, Selected SNR
    '''
    while True:
        code_type = input('Please choose one of these coding types (index only [1-4]): \n '
                          '1. LDPC \n 2. Convolutional \n 3. Turbo \n 4. Polar \n \n')

        codes = {
            1: {'name': 'LDPC', 'Available_Channels': {1: {'ch': 'AWGN', 'snr_list': [-2, 0]}, 2: {'ch' : 'SinglePath', 'snr_list': [20, 0]}, 3: {'ch' : 'MultiPath', 'snr_list': [20,0]}}, 'classes_list': ['R12', 'R23', 'R34', 'R56']},
            2: {'name': 'Convolutional', 'Available_Channels': {1: {'ch': 'AWGN', 'snr_list': [2, 0]}, 2: {'ch' : 'MultiPath', 'snr_list': [20, 0]}}, 'classes_list': ['C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7']},
            3: {'name': 'Turbo', 'Available_Channels': {1: {'ch': 'AWGN', 'snr_list': [-2, -6]}, 2: {'ch': 'SinglePath', 'snr_list': [20, 0]}, 3: {'ch' : 'MultiPath', 'snr_list': [20,0]}},'classes_list': ['T1', 'T2', 'T3', 'T4']},
            4: {'name': 'Polar', 'Available_Channels': {1: {'ch': 'AWGN', 'snr_list': [6, -2]}, 2: {'ch': 'SinglePath', 'snr_list': [20,0]}, 3: {'ch' : 'MultiPath', 'snr_list': [20,0]}}, 'classes_list': ['P1', 'P2', 'P3', 'P4']}
        }

        try:
            code = codes[int(code_type)]
            break

        except:
            print('Invalid Input: Please enter only the index of the option')

    while True:

        txt = 'Please choose one of the following channel models (index only): \n'

        # for i, channel in zip(code['Available_Channels'].keys(), code['Available_Channels'].values()):
        for i in code['Available_Channels'].keys():
            channel = code['Available_Channels'][i]['ch']
            txt += '{}. {}'.format(i, channel)
            txt += '\n'

        channel_type = input(txt)

        try:
            channel = code['Available_Channels'][int(channel_type)]['ch']
            snr_list = code['Available_Channels'][int(channel_type)]['snr_list']
            break
        except:
            print('Invalid Input: Please enter only the index of the option')


    while True:

        txt = 'Please choose one of the following SNRs (index only): \n'

        for i, snr in enumerate(snr_list):
            txt += '{}. {}dB'.format(i+1, snr)
            txt += '\n'

        snr_index = input(txt)

        try:
            snr = snr_list[int(snr_index)-1]
            break
        except:
            print('Invalid Input: Please enter only the index of the option')

    print(code['name'])
    print(code['classes_list'])
    print(channel)
    print(snr)
    return code['name'], code['classes_list'], channel, snr

def Evaluation(model_name, test_filename, class_list, snr_db) :
    '''
    :param model_name: LDPC/LDPC_AWGN.model
    :param test_filename: LDPC/Testset-AWGN/2dB.npz
    :param class_list: [R12, R23, R34, R56]
    :param snr_db
    :return:
    '''

    print('Evaluating Model ...')

    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID";
    os.environ['CUDA_VISIBLE_DEVICES'] = '2'

    model = load_model(model_name)

    Val = np.load(test_filename)
    X = Val['X']
    Y = Val['Y']

    SamplePerRate = int(X.shape[0] / len(class_list))

    for i, code_class in enumerate(class_list):
        X_val = X[SamplePerRate * i: SamplePerRate * (i + 1)]
        Y_val = Y[SamplePerRate * i: SamplePerRate * (i + 1)]

        Result = model.evaluate(X_val, Y_val, verbose=0)

        print('*********************')
        print('SNR = %s dB' % snr_db)
        print('Config = %s' % code_class)
        print('Test accuracy:', Result[1])
        print('*********************')


if __name__ == '__main__':
    code, class_list, channel, snr = Menu()

    test_filename = '{}/Testset-{}/{}dB.npz'.format(code, channel, snr)
    model_name = '{}/{}-{}.model'.format(code, code, channel)

    print(test_filename)
    print(model_name)

    Evaluation(model_name, test_filename, class_list, snr)


